// Importing required modules
import {
  select,
  json,
  geoPath,
  geoEquirectangular,
  scaleOrdinal,
} from 'd3';
import { feature } from 'topojson';

// Selecting SVG element
const svg = select('svg');

// Define colors for countries
const colorScale = scaleOrdinal()
  .domain([
    'USA',
    'Canada',
    'Mexico',
    'Russia',
    'China',
    'India',
    'Brazil',
    'Australia',
    'United Kingdom',
    'Germany',
    'France',
    'Spain',
    'Italy',
    'South Africa',
    'Japan',
  ])
  .range([
    '#E0BBE4',
    '#957DAD',
    '#D291BC',
    '#FEC8D8',
    '#FFDFD3',
    '#D9AFD9',
    '#DBD4B4',
    '#FFE1A8',
    '#FF7F0E',
    '#2CA02C',
    '#17BECF',
    '#9EDAE5',
    '#1F77B4',
    '#FF9896',
    '#9467BD',
  ]);

// Creating projection and path generator
const projection = geoEquirectangular();
const pathGenerator = geoPath().projection(projection);

// Adding path element for the sphere
svg
  .append('path')
  .attr('class', 'earth')
  .attr('d', pathGenerator({ type: 'Sphere' }));

// Loading world map data
json(
  'https://cdn.jsdelivr.net/npm/world-atlas@2/countries-110m.json',
).then((data) => {
  const countries = feature(data, data.objects.countries);

  // Adding country paths with different colors
  svg
    .selectAll('.country')
    .data(countries.features)
    .enter()
    .append('path')
    .attr('class', 'country')
    .attr('d', pathGenerator)
    .style('fill', (d) => colorScale(d.properties.name)) // Assigning color based on country name
    .style('stroke', '#000') // Adding black border
    .on('mouseover', function () {
      // Highlight country on mouseover
      select(this).style('fill', 'yellow');
    })
    .on('mouseout', function () {
      // Reset color on mouseout
      select(this).style('fill', (d) =>
        colorScale(d.properties.name),
      );
    });

  // Adding country names inside the country shapes
  svg
    .selectAll('.country-label')
    .data(countries.features)
    .enter()
    .append('text')
    .attr('class', 'country-label')
    .attr('text-anchor', 'middle') // Center align the text
    .attr('dy', '0.25em') // Adjust vertical position
    .attr(
      'transform',
      (d) => `translate(${pathGenerator.centroid(d)})`,
    )
    .style('fill', 'black') // Setting text color
    .style('font-size', '4px') // Increasing font size for better readability
    .style('font-family', 'Arial, sans-serif') // Setting font family
    .text((d) => d.properties.name); // Using country name from GeoJSON properties
});
